#
# Unit Tests for numtypes
#
#

import os
import unittest
import pyutilib.math


class NumTypesDebug(unittest.TestCase):

    def test_infinity1(self):
        # Check that infinity is defined appropriately
        if 1 > pyutilib.math.infinity:
            self.fail("test_infinity")
        if not ((1.0 / pyutilib.math.infinity) == 0.0):
            self.fail("test_infinity - 1/infinity is not zero")
        if pyutilib.math.is_finite(pyutilib.math.infinity):
            self.fail("test_infinity - infinity is finite")
        if pyutilib.math.is_finite(-pyutilib.math.infinity):
            self.fail("test_infinity - -infinity is finite")
        if not pyutilib.math.is_finite(1.0):
            self.fail("test_infinity - 1.0 is not finite")
        if pyutilib.math.is_finite(None):
            self.fail("test_infinity - None is not infinite")
        if pyutilib.math.is_finite('foo'):
            self.fail("test_infinity - 'foo' is not infinite")

    def test_nan(self):
        # Check that nan is defined appropriately
        if not type(pyutilib.math.nan) is type(1.0):
            self.fail("test_nan")
        if not pyutilib.math.is_nan(pyutilib.math.infinity /
                                    pyutilib.math.infinity):
            self.fail("test_nan - infinity/infinity is not NaN")
        if pyutilib.math.is_nan(1.0 / pyutilib.math.infinity):
            self.fail("test_nan - 1.0/infinity is NaN")
        if pyutilib.math.is_nan(None):
            self.fail("test_nan - None is not NaN")
        if pyutilib.math.is_nan('foo'):
            self.fail("test_nan - 'foo' is not NaN")


if __name__ == "__main__":
    unittest.main()



#
# Unit Tests for util/math
#
#

import unittest
import pyutilib.math


class MathDebug(unittest.TestCase):

    def test_isint(self):
        # Verify that isint() identifies ints
        if pyutilib.math.isint([1, 2]):
            self.fail("test_isint - thought that a list was an integer")
        if pyutilib.math.isint("a"):
            self.fail("test_isint - thought that a string was integer")
        if not pyutilib.math.isint(" 1 "):
            self.fail(
                "test_isint - thought that an integer string was not an integer")
        if not pyutilib.math.isint(" 1.0 "):
            self.fail(
                "test_isint - thought that an integer float string was not an integer")
        if pyutilib.math.isint(" 1.1 "):
            self.fail("test_isint - thought that a float string was an integer")
        if pyutilib.math.isint(1.1):
            self.fail("test_isint - thought that a float was integer")
        if not pyutilib.math.isint(1.0):
            self.fail(
                "test_isint - thought that an integer float was not an integer")
        if not pyutilib.math.isint(1):
            self.fail("test_isint - thought that an integer was not an integer")

    def test_argmax(self):
        # Test that argmax works
        a = [0, 1, 2, 3]
        self.assertEqual(a[pyutilib.math.argmax(a)], a[3])
        a = [3, 2, 1, 0]
        self.assertEqual(a[pyutilib.math.argmax(a)], a[0])

    def test_argmin(self):
        # Test that argmin works
        a = [0, 1, 2, 3]
        self.assertEqual(a[pyutilib.math.argmin(a)], a[0])
        a = [3, 2, 1, 0]
        self.assertEqual(a[pyutilib.math.argmin(a)], a[3])

    def test_mean(self):
        # Verify that mean() works
        self.assertEqual(pyutilib.math.mean((1, 2, 3)), 2.0)
        try:
            val = pyutilib.math.mean([])
            self.fail("test_mean - should have failed with an empty list")
        except ArithmeticError:
            pass

    def test_median(self):
        # Verify that median() works
        self.assertEqual(pyutilib.math.median((1, 2, 3)), 2.0)
        self.assertEqual(pyutilib.math.median((2,)), 2.0)
        self.assertEqual(pyutilib.math.median((1, 2, 3, 4)), 2.5)
        try:
            val = pyutilib.math.median([])
            self.fail("test_median - should have failed with an empty list")
        except ArithmeticError:
            pass

    def test_factorial(self):
        # Verify that factorial() works
        self.assertEqual(pyutilib.math.factorial(0), 1)
        self.assertEqual(pyutilib.math.factorial(1), 1)
        self.assertEqual(pyutilib.math.factorial(2), 2)
        self.assertEqual(pyutilib.math.factorial(3), 6)
        self.assertEqual(pyutilib.math.factorial(4), 24)
        try:
            val = pyutilib.math.factorial(-1)
            self.fail(
                "test_factorial - should have failed with a negative value")
        except ArithmeticError:
            pass

    def test_perm(self):
        # Verify that perm() works
        self.assertEqual(pyutilib.math.perm(7, 1), 7)
        self.assertEqual(pyutilib.math.perm(7, 2), 21)


if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
"""
This is a convenience class that configures the PyUtilib Component Architecture
for a named application.  This class registers and activates a variety of
plugins that are commonly used in simple applications.
"""

import pyutilib.component.core
import pyutilib.component.config
import os


class SimpleApplication(object):

    def __init__(self, name, filename=None):
        self.name = name
        if filename is None:
            self.filename = 'config.ini'
        else:
            self.filename = filename
        self.env = pyutilib.component.core.PluginGlobals.add_env(self.name)
        self.config = pyutilib.component.config.Configuration(filename)
        self._env_config = pyutilib.component.config.EnvironmentConfig(name)
        self._env_config.options.path = os.getcwd()
        self.logger = pyutilib.component.config.LoggingConfig(name)
        self._egg_plugin = pyutilib.component.core.PluginFactory(
            "EggLoader", namespace=name, env='pca')

    def configure(self, filename):
        """Load a configuration file, and update options"""
        self.config.load(filename)

    def save_configuration(self, filename):
        """Save a configuration file"""
        self.config.save(filename)

    def exit(self):
        """Perform cleanup operations"""
        self.logger.shutdown()

    def log(self, message):
        """Generate logging output"""
        self.logger.log(message)

    def flush(self):
        """Flush the log output"""
        self.logger.flush()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import inspect
import logging
import os
import sys

from pyutilib.misc.indent_io import StreamIndenter

class LogHandler(logging.Handler):

    def __init__( self, base='', stream=sys.stdout, 
                  level=logging.NOTSET, verbosity=None ):
        if verbosity is not None:
            self.verbosity = verbosity
        else:
            self.verbosity = lambda: True
        logging.Handler.__init__(self, level=level)

        self.stream = stream
        self.basepath = base

    def emit(self, record):
        level = record.levelname
        msg = record.getMessage()

        if self.verbosity():
            filename = record.pathname  # file path
            lineno = record.lineno
            try:
                function = record.funcName
            except AttributeError:
                function = '(unknown)'
            if self.basepath and filename.startswith(self.basepath):
                filename = '[base]' + filename[len(self.basepath):]

            self.stream.write(
                '%s: "%s", %d, %s\n' %
                ( level, filename, lineno, function.strip(), ))
            StreamIndenter(self.stream, "\t").write(msg.strip() + "\n")
        else:
            lines = msg.strip().splitlines(False)
            self.stream.write('%s: %s\n' % (
                level, '\n\t'.join(lines) ))

# Set up default logging for PyUtilib
pyutilib_base = os.path.normpath(os.path.join(
    os.path.dirname( os.path.abspath(
        inspect.getframeinfo(inspect.currentframe()).filename)),
    '..', '..', '..'))

logger = logging.getLogger('pyutilib')
logger.setLevel(logging.WARNING)
logger.addHandler(
    LogHandler(
        pyutilib_base, verbosity=lambda: logger.isEnabledFor(logging.DEBUG)))



#
# Unit Tests for SparseMapping
#
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__)) + os.sep
import pyutilib.th as unittest
import pyutilib.misc


class Test(unittest.TestCase):

    def setUp(self):
        if sys.version_info[:2] < (2, 6):
            self.skipTest(
                "SparseMapping not fully defined for Python 2.4 and 2.5")

    def test1(self):
        # Validate behavior for empty sparse map
        smap = pyutilib.misc.SparseMapping()
        self.assertEqual(len(smap), 0)

    def test2(self):
        # Validate behavior for sparse map with specified values
        smap = pyutilib.misc.SparseMapping(a=1, b=2)
        self.assertEqual(len(smap), 2)
        self.assertTrue('a' in smap)
        data = [key for key in smap]
        self.assertEqual(sorted(data), ['a', 'b'])
        try:
            smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        self.assertEqual(smap['a'], 1)
        try:
            del smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        del smap['b']
        self.assertEqual(len(smap), 1)
        smap['b'] = 3
        self.assertEqual(smap['b'], 3)

    def test3(self):
        # Validate behavior for sparse map with specified values and domain
        smap = pyutilib.misc.SparseMapping(a=1, b=2, index=['a', 'b', 'z'])
        self.assertEqual(len(smap), 2)
        self.assertTrue('a' in smap)
        data = [key for key in smap]
        self.assertEqual(sorted(data), ['a', 'b'])
        try:
            smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        self.assertEqual(smap['a'], 1)
        try:
            del smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        del smap['b']
        self.assertEqual(len(smap), 1)
        smap['b'] = 3
        self.assertEqual(len(smap), 2)
        self.assertEqual(smap['b'], 3)

    def test4(self):
        # Validate behavior for sparse map with specified values and domain and default value
        smap = pyutilib.misc.SparseMapping(
            a=1, b=2, index=['a', 'b', 'z'], default=0)
        self.assertEqual(len(smap), 3)
        self.assertTrue('a' in smap)
        data = [key for key in smap]
        self.assertEqual(sorted(data), ['a', 'b', 'z'])
        try:
            smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        self.assertEqual(smap['a'], 1)
        try:
            del smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        del smap['b']
        self.assertEqual(len(smap), 3)
        smap['b'] = 3
        self.assertEqual(len(smap), 3)
        self.assertEqual(smap['b'], 3)
        self.assertEqual(smap['z'], 0)

    def test6(self):
        # Validate behavior for sparse map with specified values and default value
        smap = pyutilib.misc.SparseMapping(a=1, b=2, default=0)
        self.assertEqual(len(smap), 2)
        self.assertTrue('a' in smap)
        data = [key for key in smap]
        self.assertEqual(sorted(data), ['a', 'b'])
        self.assertEqual(smap['c'], 0)
        self.assertEqual(smap['a'], 1)
        try:
            del smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        del smap['b']
        self.assertEqual(len(smap), 1)
        smap['b'] = 3
        self.assertEqual(len(smap), 2)
        self.assertEqual(smap['b'], 3)
        self.assertEqual(smap['z'], 0)

    def test5(self):
        # Validate behavior for sparse map with specified values and 'within' option
        smap = pyutilib.misc.SparseMapping(a=1, b=2, within=range(10))
        self.assertEqual(len(smap), 2)
        self.assertTrue('a' in smap)
        data = [key for key in smap]
        self.assertEqual(sorted(data), ['a', 'b'])
        try:
            smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        self.assertEqual(smap['a'], 1)
        try:
            del smap['c']
            self.fail("Expected KeyError")
        except KeyError:
            pass
        del smap['b']
        self.assertEqual(len(smap), 1)
        try:
            smap['b'] = -1
            self.fail("Expected ValueError")
        except ValueError:
            pass
        smap['b'] = 3
        self.assertEqual(smap['b'], 3)


if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import sys
try:
    import nose
    import nose.plugins
except ImportError:
    print("This testing utility requires the 'nose' Python testing tool")
    sys.exit(1)
try:
    import nosexunit
    import nosexunit.plugin
    nose_xunit_installed = True
except ImportError:
    nose_xunit_installed = False
try:
    import coverage
    import nose.plugins.cover
    coverage_installed = True
except ImportError:
    coverage_installed = False

import re
import logging
import os
import os.path
import glob
import copy
from optparse import OptionParser

# Ignore exceptions that occur during a logging statement
logging.raiseExceptions = 0


class UnwantedPackagePlugin(nose.plugins.Plugin):
    enabled = True
    name = "unwanted-package"

    def __init__(self, unwanted):
        self.unwanted = unwanted

    def configure(self, options, conf):
        pass

    def wantDirectory(self, dirname):
        want = None
        if os.path.basename(dirname) in self.unwanted:
            want = False
        return want


def generate_options(package, packages, subpackages, filter, options, testdir,
                     xunitdir, dirname):
    if not type(packages) is list:
        tmp = [packages]
    else:
        tmp = packages
    newargs = []
    if options.xunit:
        newargs = ['--with-xunit', '--xunit-file', xunitdir + os.sep + 'TEST-' +
                   package + '.xml']
        #if coverage_installed:
    #newargs = newargs + ['--source-folder', '.']
    if coverage_installed:
        newargs = newargs + ['--with-coverage', '--cover-erase',
                             '--cover-inclusive', '--cover-package', package]
        if options.filtering or options.filtering_all:
            for item in filter:
                newargs.append('--cover-filter=' + item)
            if len(tmp) > 0:
                for item in subpackages:
                    if not item in tmp:
                        newargs.append('--cover-filter=' + package + '.' + item)
    if options.verbose:
        newargs = newargs + ['--verbosity=3']
    newargs = newargs + ['--where', dirname]
    return newargs


def main(argv, package, subpackages, filter, testdir, dirname="test"):

    parser = OptionParser()
    parser.add_option(
        "--no-xunit",
        help="Disable generation of XUnit XML summary",
        action="store_false",
        dest="xunit",
        default=True)
    parser.add_option(
        "-v",
        "--verbose",
        help="Provide verbose output",
        action="store_true",
        dest="verbose",
        default=False)
    parser.add_option(
        "-f",
        "--filter",
        help="Enable filtering of coverage tests with a customized nose installation",
        action="store_true",
        dest="filtering",
        default=False)
    parser.add_option(
        "-F",
        "--filter-all",
        help="Apply nose to all packages, filtering each coverage test",
        action="store_true",
        dest="filtering_all",
        default=False)
    parser.usage = "runtests [options] <package> [...]"
    parser.description = "A utility to run Pyomo tests.  A particular feature of this tool is the ability to filter coverage information, using a customized installation of the nose utility."
    parser.epilog = """The customized nose utility that supports filtering can be obtained from Bill Hart (wehart@sandia.gov)."""
    #
    # Process argv list
    #
    (options, args) = parser.parse_args(args=argv)
    #
    # Preprocess arguments, to eliminate trailing '/' or '\\' characters, which a user
    # might add using tab completion.
    #
    for i in range(0, len(args)):
        if args[i][-1] == os.sep:
            args[i] = args[i][:-1]
    #
    # Verify that the arguments are valid subpackages
    #
    unwanted = copy.copy(subpackages)
    for arg in args[1:]:
        if not arg in subpackages:
            print("ERROR: no subpackage '" + arg)
            sys.exit(1)
        unwanted.remove(arg)
    if unwanted == subpackages:
        unwanted = []
    #
    # Remove coverage files from previous 'runtests' executions
    #
    coverage_file = testdir + os.sep + dirname + os.sep + ".coverage"
    if os.path.exists(coverage_file):
        os.remove(coverage_file)
    os.environ["COVERAGE_FILE"] = coverage_file
    #for file in glob.glob("*/.coverage"):
    #os.remove(file)
    #for file in glob.glob("*/*/.coverage"):
    #os.remove(file)
    #
    # Remove XML files from previous 'runtests' executions
    #
    xunitdir = testdir + os.sep + dirname + os.sep + "xunit"
    if os.path.exists(xunitdir):
        for file in glob.glob(xunitdir + os.sep + "TEST*.xml"):
            os.remove(file)
    else:
        os.mkdir(xunitdir)

    plugins = [UnwantedPackagePlugin(unwanted)]
    #if nose_xunit_installed and options.xunit:
    #plugins.append(nosexunit.plugin.NoseXUnit())
    #if coverage_installed:
    #plugins.append(nose.plugins.cover.Coverage())
    #plugins.append(nose.plugins.xunit.Xunit())
    if not options.filtering_all:
        newargs = ['runtests'] + generate_options(
            package, args[1:], subpackages, filter, options, testdir, xunitdir,
            dirname) + args[1:]
        os.chdir(testdir)
        if options.verbose:
            print("Nose Arguments:", newargs)
            print("Test Dir:", testdir)
            if len(args) > 1:
                print("Test Packages:",)
                for arg in args[1:]:
                    print(arg,)
                print("")
            if len(unwanted) > 1:
                print("Ignored Packages:", unwanted)
        sys.argv = newargs
        nose.run(argv=newargs, addplugins=plugins)
    else:
        if len(args[1:]) == 0:
            arglist = subpackages
        else:
            arglist = args[1:]
        for arg in arglist:
            newargs = ['runtests'] + generate_options(arg, options) + [arg]
            print("Package Coverage Tests: " + arg)
            tmp = sys.stdout
            os.chdir(testdir)
            if options.verbose:
                print("Nose Arguments:", newargs)
                print("Test Dir:", testdir)
                if len(args) > 1:
                    print("Test Packages:",)
                    for arg in args[1:]:
                        print(arg,)
                    print("")
                if len(unwanted) > 1:
                    print("Ignored Packages:", unwanted)
            nose.run(argv=newargs, addplugins=plugins)
            sys.stdout = tmp
    #
    # Rename xUnit files and insert the package information
    #
    if os.path.exists(xunitdir):
        p = re.compile('^.*TEST-(.+)\.xml')
        for file in glob.glob(xunitdir + os.sep + "TEST-*.xml"):
            oldname = p.match(file).group(1)
            if oldname == "test":
                suffix = ""
            else:
                suffix = "." + oldname

            FILE = open(file, "r")
            text0 = "".join(FILE.readlines())
            FILE.close()
            ptmp = re.compile("testsuite name=\"nosetests\"")
            text1 = ptmp.sub("testsuite name=\"" + package + "\"", text0)
            #ptmp = re.compile("classname=\""+oldname)
            #text2 = ptmp.sub("classname=\""+package+suffix, text1)
            #FILE = open(xunitdir+os.sep+"TEST-"+package+suffix+".xml","w")
            FILE = open(file, "w")
            FILE.write(text1)
            FILE.close()
            #os.remove(file)



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________


def median(mylist):
    """
    Returns the median value of a list
    """
    mylist = list(mylist)
    mylist.sort()
    if (len(mylist) == 0):
        raise ArithmeticError(
            "Attempting to compute the median of a zero-length list")
    elif (len(mylist) == 1):
        return mylist[0]
    elif (divmod(len(mylist), 2)[1] == 1):
        return mylist[(len(mylist) - 1) // 2]
    ndx = len(mylist) // 2
    return (mylist[ndx - 1] + mylist[ndx]) / 2.0



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

# This software is adapted from the Trac software (specifically, the trac.core
# module.  The Trac copyright statement is included below.

__all__ = ['ImportLoader']

from glob import glob
import imp
import re
import os
import sys
import logging

from pyutilib.component.config import ManagedSingletonPlugin
from pyutilib.component.core import implements, ExtensionPoint, IIgnorePluginWhenLoading, IPluginLoader, Plugin


class ImportLoader(ManagedSingletonPlugin):
    """Loader that looks for Python source files in the plugins directories,
    which simply get imported, thereby registering them with the component
    manager if they define any components.
    """

    ep_services = ExtensionPoint(IIgnorePluginWhenLoading)

    implements(IPluginLoader)

    def load(self, env, search_path, disable_re, name_re):
        generate_debug_messages = __debug__ and env.log.isEnabledFor(
            logging.DEBUG)
        env.log.info('Loading plugins with ImportLoader')
        for path in search_path:
            plugin_files = glob(os.path.join(path, '*.py'))
            #
            # Note: for reproducibility, this fixes the order that
            # files are loaded
            #
            for plugin_file in sorted(plugin_files):
                #print("ImportLoader:",plugin_file)
                #
                # Load the module
                #
                module = None
                plugin_name = os.path.basename(plugin_file[:-3])
                if plugin_name not in sys.modules and name_re.match(
                        plugin_name):
                    try:
                        module = imp.load_source(plugin_name, plugin_file)
                        if generate_debug_messages:
                            env.log.debug('Loading file plugin %s from %s' % \
                                  (plugin_name, plugin_file))
                    except Exception:
                        e = sys.exc_info()[1]
                        env.log.error(
                            'Failed to load plugin from %s',
                            plugin_file,
                            exc_info=True)
                        env.log.error('Load error: %r' % str(e))
                #
                # Disable singleton plugins that match
                #
                if not module is None:
                    if not disable_re.match(plugin_name) is None:
                        if generate_debug_messages:
                            env.log.debug('Disabling services in module %s' %
                                          plugin_name)
                        for item in dir(module):
                            #
                            # This seems like a hack, but
                            # without this we can disable pyutilib
                            # functionality!
                            #
                            flag = False
                            for service in ImportLoader.ep_services:
                                if service.ignore(item):
                                    flag = True
                                    break
                            if flag:
                                continue

                            cls = getattr(module, item)
                            try:
                                is_instance = isinstance(cls, Plugin)
                            except TypeError:  #pragma:nocover
                                is_instance = False
                            try:
                                is_plugin = issubclass(cls, Plugin)
                            except TypeError:
                                is_plugin = False
                            try:
                                is_singleton = not (cls.__instance__ is None)
                            except AttributeError:  #pragma:nocover
                                is_singleton = False
                            if is_singleton and is_plugin:
                                if generate_debug_messages:
                                    env.log.debug('Disabling service %s' % item)
                                cls.__instance__._enable = False
                            if is_instance:
                                if generate_debug_messages:
                                    env.log.debug('Disabling service %s' % item)
                                cls._enable = False
                    elif generate_debug_messages:
                        env.log.debug('All services in module %s are enabled' %
                                      plugin_name)

# Copyright (C) 2005-2008 Edgewall Software
# Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.org/wiki/TracLicense.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://trac.edgewall.org/log/.
#
# Author: Christopher Lenz <cmlenz@gmx.de>



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
#

__all__ = ['ITestDriver', 'TestDriverFactory', 'ITestParser', 'TestDriverBase']

from pyutilib.component.core import Interface, implements, Plugin, CreatePluginFactory


class ITestParser(Interface):

    def load_test_config(self, filename):
        pass  #pragma:nocover

    def print_test_config(self, repn):
        pass  #pragma:nocover


class ITestDriver(Interface):

    def setUpClass(self, cls, options):
        """Set-up the class that defines the suite of tests"""

    def tearDownClass(self, cls, options):
        """Tear-down the class that defines the suite of tests"""

    def setUp(self, testcase, options):
        """Set-up a single test in the suite"""

    def tearDown(self, testcase, options):
        """Tear-down a single test in the suite"""

    def run_test(self, testcase, name, options):
        """Execute a single test in the suite"""


class TestDriverBase(Plugin):

    implements(ITestDriver)

    def setUpClass(self, cls, options):
        """Set-up the class that defines the suite of tests"""

    def tearDownClass(self, cls, options):
        """Tear-down the class that defines the suite of tests"""

    def setUp(self, testcase, options):
        """Set-up a single test in the suite"""

    def tearDown(self, testcase, options):
        """Tear-down a single test in the suite"""

    def run_test(self, testcase, name, options):
        """Execute a single test in the suite"""


TestDriverFactory = CreatePluginFactory(ITestDriver)



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import pyutilib.misc
from pyutilib.component.core import Interface, Plugin, implements
from pyutilib.component.config import ExecutableOption, declare_option


class IExternalExecutable(Interface):
    """Interface for plugins that define an external executable"""

    def get_path(self):
        """Returns a string that is the path of the executable"""


class ExternalExecutable(Plugin):

    implements(IExternalExecutable, service=True)

    def __init__(self, **kwds):
        if 'doc' in kwds:
            self.exec_doc = kwds["doc"]
        else:
            self.exec_doc = ""
        if 'name' in kwds:
            self.name = kwds['name']
            declare_option(
                kwds['name'],
                local_name="executable",
                section="executables",
                default=None,
                doc=self.exec_doc,
                cls=ExecutableOption)
        else:
            raise PluginError("An ExternalExectuable requires a name")
        if 'path' in kwds:
            self.path = kwds['path']
        else:
            self.path = None
        if 'validate' in kwds:
            self.validate = kwds['validate']
        else:
            self.validate = None
        self.find_executable()

    def find_executable(self):
        if not self.path is None:
            self.exec_default = self.path
        else:
            self.exec_default = pyutilib.misc.search_file(
                self.name,
                implicitExt=pyutilib.misc.executable_extension,
                executable=True,
                validate=self.validate)

    def enabled(self):
        return self._enable and ((self.executable is not None) or
                                 (self.exec_default is not None))

    def get_path(self):
        if not self.enabled():
            return None
        tmp = self.executable
        if tmp is None:
            return self.exec_default
        return tmp



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
"""
A class for interacting with an Excel spreadsheet.
"""

#
# Imports
#
import os
import sys

import xlrd

from pyutilib.excel.base import ExcelSpreadsheet_base


class ExcelSpreadsheet_xlrd(ExcelSpreadsheet_base):

    def can_read(self):
        return True

    def can_write(self):
        return False

    def can_calculate(self):
        return False

    def __init__(self, filename=None, worksheets=(1,), default_worksheet=1):
        """
        Constructor.
        """
        self.wb = None
        self.xlsfile = None
        self._ws = {}
        if filename is not None:
            self.open(filename, worksheets, default_worksheet)

    def open(self, filename, worksheets=(1,), default_worksheet=1):
        """
        Initialize this object from a file.
        """
        #
        # Set the excel spreadsheet name
        #
        self.xlsfile = filename
        #
        # Start the excel spreadsheet
        #
        self.wb = xlrd.open_workbook(self.xlsfile)
        self.worksheets = set(worksheets)
        self._ws = {}
        for wsid in worksheets:
            if type(wsid) is int:
                self._ws[wsid] = self.wb.sheet_by_index(wsid - 1)
            else:
                self._ws[wsid] = self.wb.sheet_by_name(wsid)
        self.default_worksheet = default_worksheet

    def ws(self):
        """ The active worksheet """
        return self._ws[self.default_worksheet]

    def __del__(self):
        """
        Close the spreadsheet when deleting this object.
        """
        if self is None:
            return
        self.close()

    def close(self):
        """
        Close the spreadsheet
        """
        if self is None:  #pragma:nocover
            return
        if self._ws is None:
            return
        self._ws = {}

    def activate(self, name):
        """ Activate a specific sheet """
        if name is None:
            return
        if not name in self._ws:
            raise ValueError("Cannot activate a missing sheet with xlrd")
        self.default_worksheet = name

    def calc_iterations(self, val=None):
        raise ValueError(
            "ExcelSpreadsheet calc_iterations() is not supported with xlrd")

    def max_iterations(self, val=None):
        raise ValueError(
            "ExcelSpreadsheet max_iterations() is not supported with xlrd")

    def calculate(self):
        """
        Perform calculations in a spreadsheet
        """
        raise ValueError(
            "ExcelSpreadsheet calculate() is not supported with xlrd")

    def set_array(self, row, col, val, wsid=None):
        """
        Set an array of cells to a given value
        """
        raise IOError("Cannot write to ranges with xlrd")

    def get_array(self, row, col, row2, col2, wsid=None, raw=False):
        """
        Return a range of cells
        """
        return self.get_range(
            (self.ws().Cells(row, col), self.ws().Cells(row2, col2)), wsid, raw)

    def set_range(self, rangename, val, wsid=None):
        """
        Set a range with a given value (or set of values)
        """
        raise IOError("Cannot write to ranges with xlrd")

    def get_column(self, colname, wsid=None, raw=False, contiguous=False):
        """
        Select the values of a column.
        This ignores blank cells at the top and bottom of the column.

        If contiguous is False, a list is returned with all cell values
        starting from the first non-blank cell until the last non-blank cell.

        If contiguous if True, a list is returned with all cell values
        starting from the first non-blank cell until the first blank cell.
        """
        raise IOError("Cannot get a named column with xlrd")

    def get_range(self, rangename, wsid=None, raw=False):
        """
        Get values for a specified range
        """
        rangeid = self._range(rangename)
        if not rangeid is None:
            return self._get_range_data(rangeid, raw)

    def _get_range_data(self, _range, raw):
        """
        Return data for the specified range.
        """
        sheet, rowxlo, rowxhi, colxlo, colxhi = _range.area2d()
        if (rowxhi - rowxlo) == 1 and (colxhi - colxlo) == 1:
            return self._translate(sheet.cell(rowxlo, colxlo))
        else:
            #
            # If the range is a column or row of data, then return a list of values.
            # Otherwise, return a tuple of tuples
            #
            ans = []
            for i in range(rowxhi - rowxlo):
                col = []
                for j in range(colxhi - colxlo):
                    val = self._translate(sheet.cell(i + rowxlo, j + colxlo))
                    col.append(val)
                if len(col) == 1:
                    ans.append(col[0])
                else:
                    ans.append(list(col))
            return list(ans)

    def get_range_nrows(self, rangename, wsid=None):
        """
        Get the number of rows in a specified range
        """
        _range = self._range(rangename)
        sheet, rowxlo, rowxhi, colxlo, colxhi = _range.area2d()
        return rowxhi - rowxlo

    def get_range_ncolumns(self, rangename, wsid=None):
        """
        Get the number of columns in a specified range
        """
        _range = self._range(rangename)
        sheet, rowxlo, rowxhi, colxlo, colxhi = _range.area2d()
        return colxhi - colxlo

    def _range(self, rangeid, wsid=None):
        """
        Return a range for a given worksheet
        """
        self.activate(wsid)
        #
        # If rangeid is a tuple, then this is a list of arguments to pass
        # to the Range() method.
        #
        if type(rangeid) is tuple:
            return self.ws().Range(*rangeid)
        #
        # Otherwise, we assume that this is a range name.
        #
        else:
            tmp_ = self.wb.name_map.get(rangeid.lower(), None)
            if tmp_ is None:
                raise IOError("Range %s is not found" % rangeid)
            if len(tmp_) > 1:
                raise IOError("Cannot process scoped names")
            return tmp_[0]

    def _translate(self, cell):
        """
        Translate the cell value to a standard type
        """
        if cell.ctype == 0:
            return ""
        if cell.ctype == 1:
            return str(cell.value)
        if cell.ctype == 2 or cell.ctype == 3 or cell.ctype == 4:
            return cell.value
        if cell.ctype == 6:
            return None
        raise ValueError("Unexpected cell error")



import pyutilib.component.app
import pyutilib.misc
import os
import sys

currdir = sys.argv[-1] + os.sep

app = pyutilib.component.app.SimpleApplication("foo")
pyutilib.misc.setup_redirect(currdir + "summary.out")
app.config.summarize()
pyutilib.misc.reset_redirect()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ['TaskWorker', 'MultiTaskWorker', 'TaskWorkerServer']

import sys
import os
import socket
import time
import itertools
import random

from pyutilib.pyro.util import get_nameserver, using_pyro3, using_pyro4
from pyutilib.pyro.util import Pyro as _pyro
from pyutilib.pyro.util import get_dispatchers, _connection_problem

from six import advance_iterator, iteritems, itervalues
from six.moves import xrange

#
# With Pyro3 we check for a different set of errors
# in the run loop so that we don't ignore shutdown
# requests from the dispatcher
#
_worker_connection_problem = None
if using_pyro3:
    _worker_connection_problem = (_pyro.errors.TimeoutError,
                                  _pyro.errors.ConnectionDeniedError)
elif using_pyro4:
    _worker_connection_problem = _pyro.errors.TimeoutError

_worker_task_return_queue_unset = object()


class TaskWorkerBase(object):

    def __init__(self,
                 group=":PyUtilibServer",
                 host=None,
                 port=None,
                 num_dispatcher_tries=30,
                 caller_name="Task Worker",
                 verbose=False,
                 name=None):

        self._verbose = verbose
        # A worker can set this flag
        # if an error occurs during processing
        self._worker_error = False
        self._worker_shutdown = False
        self._worker_task_return_queue = _worker_task_return_queue_unset
        # sets whether or not multiple tasks should
        # be gathered from the worker queue during
        # each request for work
        self._bulk_task_collection = False

        if _pyro is None:
            raise ImportError("Pyro or Pyro4 is not available")

        # Deprecated in Pyro3
        # Removed in Pyro4
        if using_pyro3:
            _pyro.core.initClient()

        if name is None:
            self.WORKERNAME = "Worker_%d@%s" % (os.getpid(),
                                                socket.gethostname())
        else:
            self.WORKERNAME = name

        self.ns = get_nameserver(host=host, port=port, caller_name=caller_name)
        if self.ns is None:
            raise RuntimeError("TaskWorkerBase failed to locate "
                               "Pyro name server on the network!")
        print('Worker attempting to find Pyro dispatcher object...')
        cumulative_sleep_time = 0.0
        self.dispatcher = None
        for i in xrange(0, num_dispatcher_tries):
            dispatchers = get_dispatchers(group=group, ns=self.ns)
            random.shuffle(dispatchers)
            for name, uri in dispatchers:
                try:
                    if using_pyro3:
                        self.dispatcher = _pyro.core.getProxyForURI(uri)
                    else:
                        self.dispatcher = _pyro.Proxy(uri)
                        self.dispatcher._pyroTimeout = 10
                    if not self.dispatcher.register_worker(self.WORKERNAME):
                        if using_pyro4:
                            self.dispatcher._pyroRelease()
                        else:
                            self.dispatcher._release()
                        self.dispatcher = None
                    else:
                        break
                except _connection_problem:
                    self.dispatcher = None
            if self.dispatcher is not None:
                if using_pyro4:
                    self.dispatcher._pyroTimeout = None
                break
            sleep_interval = random.uniform(5.0, 15.0)
            print("Worker failed to find dispatcher object from "
                  "name server after %d attempts and %5.2f seconds "
                  "- trying again in %5.2f seconds." %
                  (i + 1, cumulative_sleep_time, sleep_interval))
            time.sleep(sleep_interval)
            cumulative_sleep_time += sleep_interval

        if self.dispatcher is None:
            raise RuntimeError(
                "Worker could not find dispatcher object - giving up")

        # There is no need to retain the proxy connection to the
        # nameserver, so free up resources on the nameserver thread
        URI = None
        if using_pyro4:
            self.ns._pyroRelease()
            URI = self.dispatcher._pyroUri
        else:
            self.ns._release()
            URI = self.dispatcher.URI

        print("Connection to dispatch server %s established "
              "after %d attempts and %5.2f seconds - "
              "this is worker: %s" %
              (URI, i + 1, cumulative_sleep_time, self.WORKERNAME))

        # Do not release the connection to the dispatcher
        # We use this functionality to distribute workers across
        # multiple dispatchers based off of denied connections

    def close(self):
        if self.dispatcher is not None:
            self.dispatcher.unregister_worker(self.WORKERNAME)
            if using_pyro4:
                self.dispatcher._pyroRelease()
            else:
                self.dispatcher._release()
        self.dispather = None

    def run(self):
        raise NotImplementedError       #pragma:nocover

class TaskWorker(TaskWorkerBase):

    def __init__(self, type=None, block=True, timeout=None, *args, **kwds):

        self.type = type
        self.block = block
        self.timeout = timeout
        # Indicates whether or not we assume that all task
        # ids are contiguous and process them as such
        self._contiguous_task_processing = False
        self._next_processing_id = None
        TaskWorkerBase.__init__(self, *args, **kwds)

    def run(self):

        print("Listening for work from dispatcher...")

        while 1:
            self._worker_error = False
            self._worker_shutdown = False
            try:
                tasks = None
                if self._bulk_task_collection:
                    tasks_ = self.dispatcher.get_tasks(
                        ((self.type, self.block, self.timeout),))
                    assert len(tasks_) == 1
                    assert self.type in tasks_
                    tasks = tasks_[self.type]
                else:
                    tasks = (self.dispatcher.get_task(
                        type=self.type, block=self.block, timeout=self.timeout),)
                assert tasks is not None
            except _worker_connection_problem as e:
                x = sys.exc_info()[1]
                # this can happen if the dispatcher is overloaded
                print("***WARNING: Connection to dispatcher server "
                      "denied\n - exception type: " + str(type(e)) +
                      "\n - message: " + str(x))
                print("A potential remedy may be to increase "
                      "PYUTILIB_PYRO_MAXCONNECTIONS in your shell "
                      "environment.")
                # sleep for a bit longer than normal, for obvious reasons
                sleep_interval = random.uniform(0.05, 0.15)
                time.sleep(sleep_interval)
            else:
                if len(tasks) > 0:
                    if self._verbose:
                        print("Collected %s task(s) from queue %s" %
                              (len(tasks), self.type))
                    results = {}
                    # process tasks in order of increasing id
                    for task in sorted(tasks, key=lambda x: x['id']):
                        if self._verbose:
                            print("Processing task with id=%s from queue %s" %
                                  (task['id'], self.type))
                        if self._contiguous_task_processing:
                            # TODO: add code to skip tasks until the next contiguous
                            #       task arrives
                            if self._next_processing_id is None:
                                self._next_processing_id = task['id']
                            if self._next_processing_id != task['id']:
                                raise RuntimeError("Got task with id=%s, expected id=%s"
                                                   % (task['id'], self._next_processing_id))
                            self._next_processing_id += 1
                        self._worker_task_return_queue = \
                            _worker_task_return_queue_unset
                        self._current_task_client = task['client']
                        task['result'] = self.process(task['data'])
                        task['processedBy'] = self.WORKERNAME
                        return_type_name = self._worker_task_return_queue
                        if return_type_name is _worker_task_return_queue_unset:
                            return_type_name = self.type
                        if self._worker_error:
                            if return_type_name not in results:
                                results[return_type_name] = []
                            task['processedBy'] = self.WORKERNAME
                            results[return_type_name].append(task)
                            print(
                                "Task worker reported error during processing "
                                "of task with id=%s. Any remaining tasks in "
                                "local queue will be ignored." %
                                (task['id']))
                            break
                        if self._worker_shutdown:
                            self.close()
                            return
                        if task['generateResponse']:
                            if return_type_name not in results:
                                results[return_type_name] = []
                            results[return_type_name].append(task)

                        if self._worker_error:
                            break

                    if len(results):
                        self.dispatcher.add_results(results)

class MultiTaskWorker(TaskWorkerBase):

    def __init__(self,
                 type_default=None,
                 block_default=True,
                 timeout_default=5,
                 *args,
                 **kwds):

        TaskWorkerBase.__init__(self, *args, **kwds)

        #
        # **NOTE: For this class 'type' is
        # a tuple (type, blocking, timeout, local)
        #
        self._num_types = 0
        self._type_cycle = None
        self.push_request_type(type_default, block_default, timeout_default)

    # Called by the run() method to get the next work type to request,
    # moves index to the next location in the cycle
    def _get_request_type(self):
        new = None
        try:
            new = advance_iterator(self._type_cycle)
        except StopIteration:
            raise RuntimeError("MultiTaskWorker has no work request types!")
        return new

    def current_type_order(self):
        """
        return the full work type list, starting from the current
        location in the cycle.
        """
        if self._num_types == 0:
            return []
        type_list = []
        for cnt in xrange(self._num_types):
            type_list.append(advance_iterator(self._type_cycle))
        return type_list

    def cycle_type_order(self):
        advance_iterator(self._type_cycle)

    def num_request_types(self):
        return self._num_types

    def clear_request_types(self):
        self._type_cycle = itertools.cycle([])
        self._num_types = 0

    def push_request_type(self, type_name, block, timeout):
        """
        add a request type to the end relative to the current cycle
        location
        """
        self._type_cycle = itertools.cycle(self.current_type_order() + \
                                           [(type_name, block, timeout)])
        self._num_types += 1

    def pop_request_type(self):
        """
        delete the last type request relative to the current cycle
        location
        """
        new_type_list = self.current_type_order()
        el = new_type_list.pop()
        self._type_cycle = itertools.cycle(new_type_list)
        self._num_types -= 1
        return el

    def run(self):

        print("Listening for work from dispatcher...")

        while 1:
            self._worker_error = False
            self._worker_shutdown = False
            try:
                tasks = self.dispatcher.get_tasks(self.current_type_order())
            except _worker_connection_problem as e:
                x = sys.exc_info()[1]
                # this can happen if the dispatcher is overloaded
                print("***WARNING: Connection to dispatcher server "
                      "denied\n - exception type: " + str(type(e)) +
                      "\n - message: " + str(x))
                print("A potential remedy may be to increase "
                      "PYUTILIB_PYRO_MAXCONNECTIONS in your shell "
                      "environment.")
                # sleep for a bit longer than normal, for obvious reasons
                sleep_interval = random.uniform(0.05, 0.15)
                time.sleep(sleep_interval)
            else:

                if len(tasks) > 0:

                    if self._verbose:
                        print("Collected %s task(s) from %s queue(s)" %
                              (sum(len(_tl)
                                   for _tl in itervalues(tasks)), len(tasks)))

                    results = {}
                    # process tasks by type in order of increasing id
                    for type_name, type_tasks in iteritems(tasks):
                        type_results = results[type_name] = []
                        for task in sorted(type_tasks, key=lambda x: x['id']):
                            self._worker_task_return_queue = \
                                _worker_task_return_queue_unset
                            self._current_task_client = task['client']
                            task['result'] = self.process(task['data'])
                            task['processedBy'] = self.WORKERNAME
                            return_type_name = self._worker_task_return_queue
                            if return_type_name is _worker_task_return_queue_unset:
                                return_type_name = type_name
                            if self._worker_error:
                                if return_type_name not in results:
                                    results[return_type_name] = []
                                results[return_type_name].append(task)
                                print(
                                    "Task worker reported error during processing "
                                    "of task with id=%s. Any remaining tasks in "
                                    "local queue will be ignored." %
                                    (task['id']))
                                break
                            if self._worker_shutdown:
                                self.close()
                                return
                            if task['generateResponse']:
                                if return_type_name not in results:
                                    results[return_type_name] = []
                                results[return_type_name].append(task)

                        if self._worker_error:
                            break

                    if len(results):
                        self.dispatcher.add_results(results)


def TaskWorkerServer(cls, **kwds):
    worker = cls(**kwds)
    if worker.ns is None:
        return
    try:
        worker.run()
    except _pyro.errors.ConnectionClosedError:
        print("Lost connection to dispatch server. "
              "Shutting down...")
    except:
        worker.close()
        raise



"""This module defines a nose plugin that allows the user to archive test data.

Use the following command-line option with nosetests ::

    nosetests --with-testdata

By default, a file named testdata.csv will be written to the working directory.
If you need to change the name or location of the file, you can set the
``--testdata-file`` option.

Here is an abbreviated version of what the CSV file might look like::

    classname,name,time
    pyutilib.th.tests.test_pyunit.Tester,test_fail,0.00291109085083
    pyutilib.th.tests.test_pyunit.Tester,test_pass,5.57899475098e-05
    pyutilib.th.tests.test_pyunit.Tester2,test_pass,0.000113964080811
    pyutilib.th.tests.test_pyunit.Tester3,test_fail,7.60555267334e-05

"""

import os
import re
from time import time

from nose.plugins.base import Plugin

from six import text_type

# Invalid CSV characters, control characters 0-31 sans \t, \n and \r
CONTROL_CHARACTERS = re.compile(r"[\000-\010\013\014\016-\037]")


def csv_safe(value):
    """Replaces invalid CSV characters with '?'."""
    return CONTROL_CHARACTERS.sub('?', value).replace(',', '_')


class TestData(Plugin):
    """This plugin archives test data in standard CSV format."""
    name = 'testdata'
    score = 2000
    encoding = 'UTF-8'
    report_file = None
    datakeys = set()

    def _timeTaken(self):
        if hasattr(self, '_timer'):
            taken = time() - self._timer
        else:
            # test died before it ran (probably error in setup())
            # or success/failure added before test started probably
            # due to custom TestResult munging
            taken = 0.0
        return taken

    def _quoteattr(self, attr):
        """Escape a CSV attribute. Value can be unicode."""
        attr = csv_safe(attr)
        if isinstance(attr, text_type):
            attr = attr.encode(self.encoding)
        return attr

    def options(self, parser, env):
        """Sets additional command line options."""
        Plugin.options(self, parser, env)
        parser.add_option(
            '--testdata-file',
            action='store',
            dest='testdata_file',
            metavar="FILE",
            default=env.get('NOSE_TESTDATA_FILE', 'testdata.csv'),
            help=("Path to CSV file to store the test data. "
                  "Default is testdata.csv in the working directory "
                  "[NOSE_TESTDATA_FILE]"))
        parser.add_option(
            '--testdata-table',
            action='store_true',
            dest='testdata_table',
            default=env.get('NOSE_TESTDATA_TABLE', False),
            help=(
                "If this option is specified, then the CSV file is "
                " formatted as a table.  By default, the format is a sparse list. "
            ))

    def configure(self, options, config):
        """Configures the testdata plugin."""
        Plugin.configure(self, options, config)
        self.config = config
        self.reportdata = []
        if options.testdata_table:
            self.format = 'table'
        else:
            self.format = 'sparse'
        if self.enabled:
            self.report_file = open(options.testdata_file, 'w')
            self.datakeys.add('time')

    def report(self, stream):
        """Writes a CSV file with test data. """
        if not os.environ.get('HUDSON_URL', None) is None:
            colprefix = 'job,build,node,'
            prefix = "%s,%s,%s," % (os.environ['JOB_NAME'],
                                    os.environ['BUILD_NUMBER'],
                                    os.environ['NODE_NAME'])
        else:
            colprefix = ''
            prefix = ''
        if self.format == 'table':
            keys = ['classname', 'name'] + sorted(list(self.datakeys))
            self.report_file.write(colprefix + ','.join(
                map(self._quoteattr, keys)) + '\n')
            for data in self.reportdata:
                tmp = []
                for key in keys:
                    tmp.append(str(data[2].get(key, '')))
                self.report_file.write(prefix + ','.join(tmp) + '\n')
            self.report_file.close()
        else:
            self.report_file.write(colprefix +
                                   'classname,name,dataname,value\n')
            for data in self.reportdata:
                for key in data[2]:
                    self.report_file.write(prefix)
                    self.report_file.write(str(data[0]) + ',' + str(data[1]))
                    self.report_file.write(',' + str(key) + ',' + str(data[2][
                        key]))
                    self.report_file.write('\n')

        if self.config.verbosity > 1:
            stream.writeln("-" * 70)
            stream.writeln("CSV: %s" % self.report_file.name)

    def startTest(self, test):
        """Initializes a timer before starting a test."""
        self._timer = time()
        test.test.testdata = {}

    def addSuccess(self, test, capt=None):
        """Add success output to test data file.
        """
        taken = self._timeTaken()
        id = test.id()
        for key in test.test.testdata:
            self.datakeys.add(key)
        classname = self._quoteattr('.'.join(id.split('.')[:-1]))
        name = self._quoteattr(id.split('.')[-1])
        test.test.testdata['time'] = taken
        self.reportdata.append((classname, name, test.test.testdata))



#
# Plugin load tests for eggs
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__)) + os.sep

import pyutilib.th as unittest
import pyutilib.subprocess

try:
    import pkg_resources
    pkg_resources_avail = True
except ImportError:
    pkg_resources_avail = False
try:
    import yaml
    yaml_available = True
except ImportError:
    yaml_available = False


class Test(pyutilib.th.TestCase):

    def test_egg1(self):
        """Load an egg for the 'project1' project.  Eggs are loaded in the 'eggs1' directory, but only the Project1 stuff is actually imported."""
        pyutilib.subprocess.run(
            [sys.executable, currdir + os.sep + "egg1.py", currdir, "json"])
        self.assertMatchesJsonBaseline(currdir + "egg1.out",
                                       currdir + "egg1.jsn")
        if yaml_available:
            pyutilib.subprocess.run(
                [sys.executable, currdir + os.sep + "egg1.py", currdir, "yaml"])
            self.assertMatchesYamlBaseline(currdir + "egg1.out",
                                           currdir + "egg1.yml")

    def test_egg2(self):
        """Load an egg for the 'project1' project.  Eggs are loaded in the 'eggs1' and 'eggs2' directories, but only the Project1 and Project 3 stuff is actually imported."""
        pyutilib.subprocess.run(
            [sys.executable, currdir + os.sep + "egg2.py", currdir, "json"])
        self.assertMatchesJsonBaseline(currdir + "egg2.out",
                                       currdir + "egg2.jsn")
        if yaml_available:
            pyutilib.subprocess.run(
                [sys.executable, currdir + os.sep + "egg2.py", currdir, "yaml"])
            self.assertMatchesYamlBaseline(currdir + "egg2.out",
                                           currdir + "egg2.yml")

# Apply class decorator explicitly, which works in Python 2.5
Test = unittest.skipIf(not pkg_resources_avail,
                       "Cannot import 'pkg_resources'")(Test)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________


class MonoState(object):
    """A class where every instance shares state."""
    _state = {}

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._state
        return self


class Singleton(object):
    """A class where the __new__() method returns the same instance."""

    def __new__(cls, *p, **k):
        if not '_the_instance' in cls.__dict__:
            cls._the_instance = object.__new__(cls)
        return cls._the_instance



import os
import re
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__)) + os.sep

import unittest
from pyutilib.component.core import Interface, PluginGlobals, ExtensionPoint, implements, Plugin
from pyutilib.component.config.options import Option, OptionError, IOption, declare_option, FileOption, IntOption, FloatOption, DictOption, BoolOption

PluginGlobals.add_env("testing.options")


class IDummyOption(Interface):
    """An interface that supports the initialization of the directory for
    options that specify files.  This is needed to correctly initialize
    relative paths for files."""


class DummyOption1(Option):
    """A test class that converts option data into float values."""

    implements(IDummyOption)

    def convert(self, value, default):
        """Conversion routine."""
        val = value[-1]
        if not val:
            return 0
        try:
            return float(val)
        except ValueError:
            raise OptionError('Expected float, got %s' % repr(value))
        except TypeError:
            raise OptionError('Expected string or float type, got %s' %
                              repr(value))


PluginGlobals.pop_env()


class TestOption(unittest.TestCase):

    def setUp(self):
        PluginGlobals.add_env("testing.options")
        PluginGlobals.clear_global_data(keys=['globals', 'a.b'])

    def tearDown(self):
        PluginGlobals.remove_env(
            "testing.options", cleanup=True, singleton=False)

    def test_init1(self):
        """Test Option construction"""
        try:
            Option()
            self.fail("expected failure")
        except OptionError:
            pass

        try:
            Option(None)
            self.fail("expected failure")
        except OptionError:
            pass

        try:
            Option("name", x=None)
            self.fail("expected failure")
        except OptionError:
            pass

    def test_init2(self):
        """Test Option construction"""
        FOO = Option("foo")
        self.assertEqual(FOO.name, "foo")
        self.assertTrue(FOO.default is None)
        self.assertTrue(FOO.section == "globals")
        self.assertTrue(FOO.section_re == None)
        self.assertTrue(FOO.__doc__ == "")

    def test_init3(self):
        """Test Option construction"""
        FOO = Option(
            name="foo", default=1, section="a", doc="b", section_re="re")
        self.assertEqual(FOO.name, "foo")
        self.assertEqual(FOO.default, 1)
        self.assertTrue(FOO.get_value() == 1)
        self.assertEqual(FOO.section, "a")
        self.assertEqual(FOO.section_re, "re")
        self.assertEqual(FOO.__doc__, "b")

    def test_set_get1(self):
        """Test set/get values"""

        class TMP_set_get1(Plugin):
            ep = ExtensionPoint(IDummyOption)
            declare_option("foo", local_name="opt", default=4, cls=DummyOption1)

        obj = TMP_set_get1()
        self.assertTrue(obj.opt == 4)
        self.assertTrue(obj.opt / 2 == 2)
        obj.opt = 6
        self.assertTrue(obj.opt / 2 == 3)
        #
        # Verify that the TMP instance has value 6
        #
        for pt in obj.ep:
            self.assertEqual(pt.get_value(), 6)

    def test_set_get2(self):
        """Test validate global nature of set/get"""

        class TMP_set_get2(Plugin):
            ep = ExtensionPoint(IOption)
            declare_option("foo", local_name="o1", default=4)
            declare_option("foo", local_name="o2", default=4)

        obj = TMP_set_get2()
        self.assertEqual(type(obj.o1), int)
        self.assertTrue(obj.o1 / 2 == 2)
        obj.o1 = 6
        self.assertTrue(obj.o1 / 2 == 3)
        self.assertTrue(obj.o2 / 2 == 3)

    def test_set_get3(self):
        """Test validate nature of set/get for instance-specific options"""

        class TMP_set_get3(Plugin):
            ep = ExtensionPoint(IOption)

            def __init__(self, section):
                declare_option("o1", section=section, default=4)

        obj1 = TMP_set_get3("sec1")
        obj2 = TMP_set_get3("sec1")
        obj3 = TMP_set_get3("sec2")
        self.assertEqual(type(obj1.o1), int)
        self.assertTrue(obj1.o1 / 2 == 2)
        self.assertTrue(obj2.o1 / 2 == 2)
        self.assertTrue(obj3.o1 / 2 == 2)
        obj1.o1 = 6
        self.assertTrue(obj1.o1 / 2 == 3)
        self.assertTrue(obj2.o1 / 2 == 3)
        self.assertTrue(obj3.o1 / 2 == 2)

    def test_repr(self):
        """Test string repn"""
        ep = ExtensionPoint(IOption)

        class TMP_repr(Plugin):
            declare_option("o1", default=4)
            declare_option("o2", section="foo", default=4)

        obj = TMP_repr()
        if re.match("\<Option \[globals\] 'o1'\>",
                    str(ep.service("o1"))) is None:
            self.fail("Expected globals:o1, but this option is %s" %
                      str(ep.service("o1")))
        self.assertFalse(
            re.match("\<Option \[globals\] 'o1'\>", str(ep.service("o1"))) is
            None)
        self.assertFalse(
            re.match("\<Option \[foo\] 'o2'\>", str(ep.service("o2"))) is None)
        self.assertEqual(ep.service("o1").get_value(), 4)
        ep.service("o1").load("o1", ["new"])
        self.assertEqual(ep.service("o1").get_value(), "new")
        ep.service("o1").load("o1", "old")
        self.assertEqual(ep.service("o1").get_value(), "old")

    def test_bool(self):
        """Test boolean"""
        ep = ExtensionPoint(IOption)

        class TMP_bool(Plugin):
            declare_option("o1", cls=BoolOption)

        obj = TMP_bool()
        pt = ep.service("o1")

        pt.load("o1", [True])
        self.assertEqual(pt.get_value(), True)

        pt.load("o1", [False])
        self.assertEqual(pt.get_value(), False)

        pt.load("o1", [1])
        self.assertEqual(pt.get_value(), True)

        pt.load("o1", [0])
        self.assertEqual(pt.get_value(), False)

        pt.load("o1", ['YES'])
        self.assertEqual(pt.get_value(), True)

        pt.load("o1", ['no'])
        self.assertEqual(pt.get_value(), False)

    def test_int(self):
        """Test int"""
        ep = ExtensionPoint(IOption)

        class TMP_int(Plugin):
            declare_option("o1", cls=IntOption)

        obj = TMP_int()
        pt = ep.service("o1")

        pt.load("o1", [-1])
        self.assertEqual(pt.get_value(), -1)

        pt.load("o1", ["-1"])
        self.assertEqual(pt.get_value(), -1)

        pt.load("o1", [[]])
        self.assertEqual(pt.get_value(), 0)

        try:
            pt.load("o1", [['a']])
            self.fail("expected error")
        except OptionError:
            pass

        try:
            pt.load("o1", ["-1.5"])
            self.fail("expected error")
        except OptionError:
            pass

    def test_float(self):
        """Test float"""
        ep = ExtensionPoint(IOption)

        class TMP_float(Plugin):
            declare_option("o1", cls=FloatOption)

        obj = TMP_float()
        pt = ep.service("o1")

        pt.load("o1", [-1.5])
        self.assertEqual(pt.get_value(), -1.5)

        pt.load("o1", ["-1.5"])
        self.assertEqual(pt.get_value(), -1.5)

        pt.load("o1", [[]])
        self.assertEqual(pt.get_value(), 0)

        try:
            pt.load("o1", [['a']])
            self.fail("expected error")
        except OptionError:
            pass

        try:
            pt.load("o1", ['a'])
            self.fail("expected error")
        except OptionError:
            pass

    def test_dict1(self):
        """Test DictOption"""
        ep = ExtensionPoint(IOption)

        class TMP_dict1(Plugin):
            declare_option("options", DictOption)
            declare_option("b")
            declare_option("c", default=3)

        obj = TMP_dict1()

        self.assertEqual(obj.options.b, None)
        self.assertEqual(obj.options.c, 3)

    def test_dict2(self):
        """Test DictOption"""
        ep = ExtensionPoint(IOption)

        class TMP_dict2(Plugin):
            declare_option("options", DictOption)
            declare_option("b", local_name="o1")
            declare_option("c", local_name="o2", default=3)

        obj = TMP_dict2()
        #
        # testing attribute set/get
        #
        obj.options.x = 3
        self.assertEqual(obj.options.x, 3)
        try:
            obj.options.xx
            self.fail("expected error")
        except OptionError:
            pass
        #
        # Testing the DictOption set/get
        #
        obj.options = {'yy': 3, 'zz': 2}
        self.assertEqual(obj.options.yy, 3)
        self.assertEqual(obj.options.zz, 2)
        #
        # Testing load
        #
        #obj.options.load('vv',1)
        #obj.options.load('uu',[1])
        #self.assertEqual(obj.options.uu,3)
        #self.assertEqual(obj.options.vv,3)

    def test_path(self):
        """Test path"""
        ep = ExtensionPoint(IOption)
        if sys.platform == "win32":
            o1_default = "C:/default"
        else:
            o1_default = "/dev//default"

        class TMP_path(Plugin):
            declare_option(
                "o1", cls=FileOption, default=o1_default, directory="/dev/null")

        obj = TMP_path()
        pt = ep.service("o1")

        pt.load("o1", [None])
        if sys.platform == "win32":
            self.assertEqual(pt.get_value(), "c:\\default")
        else:
            self.assertEqual(pt.get_value(), "/dev/default")

        if sys.platform == "win32":
            pt.load("o1", ["C:/load1"])
        else:
            pt.load("o1", ["/dev/load1"])
        if sys.platform == "win32":
            self.assertEqual(pt.get_value(), "c:\\load1")
        else:
            self.assertEqual(pt.get_value(), "/dev/load1")

        if sys.platform == "win32":
            pt.set_dir("D:/foo")
        else:
            pt.set_dir("/dev/foo")
        pt.load("o1", ["bar"])
        if sys.platform == "win32":
            self.assertEqual(pt.get_value(), "d:\\foo\\bar")
        else:
            self.assertEqual(pt.get_value(), "/dev/foo/bar")

    def test_OptionPlugin(self):
        """Test OptionPlugin"""
        ep = ExtensionPoint(IOption)

        class TMP_OptionPlugin(Plugin):
            declare_option("o1")

        obj = TMP_OptionPlugin()
        pt = ep.service("o1")

        try:
            pt.load("o1", [])
            self.fail("expected error")
        except OptionError:
            pass


if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________


def format_float(x, precision=None):
    """
    Format float in a portable manner, standardizing the
    number of digits in the exponent.
    """
    if not type(x) is float:
        raise TypeError("Argument " + str(x) + " is not a float")
    tmp = str(x)
    if not ("E" in tmp or "e" in tmp):
        return tmp
    if "+" in tmp:
        sign = "+"
    else:
        sign = "-"
    lst = tmp.split(sign)
    if lst[0] == "" and len(lst) >= 2:
        lst[1] = sign + lst[1]
        lst = lst[1:]
    if abs(x) <= 1e-100 or abs(x) >= 1e100:
        return tmp
    else:
        #
        # Use a 2-digit exponent
        #
        i = 0
        while lst[1][i] == '0':
            i += 1
        return lst[0] + sign + lst[1][i:]


def format_io(x):
    """
    Filter function for controlling the format of objects
    """
    if type(x) is float:
        return format_float(x)
    return str(x)



import pyutilib.misc
import pyutilib.th as unittest
from pyutilib.misc.visitor import SimpleVisitor, ValueVisitor


class Node(object):

    def __init__(self):
        self.children = []
        self.num = 0

    def __str__(self):
        return str(self.num)

class CountVisitor(SimpleVisitor):

    def __init__(self):
        self.count = 0

    def visit(self, node):
        self.count += 1
        node.num = self.count

    def finalize(self):
        return self.count

class CollectVisitor(SimpleVisitor):

    def __init__(self):
        self.ans = []

    def visit(self, node):
        self.ans.append(node.num)
        
    def finalize(self):
        return self.ans

class SumVisitor(ValueVisitor):

    def __init__(self):
        self.count = 0

    def visit(self, node, values):
        if values is None or len(values) == 0:
            self.count = node.num
        else:
            self.count = node.num + sum(values)
        return self.count
    
    def finalize(self, ans):
        return self.count


class Test(unittest.TestCase):

    def setUp(self):
        root = Node()
        root.children = [Node(), Node(), Node()]
        root.children[0].children = [Node(), Node(), Node()]
        root.children[0].children[0].children = [Node(), Node(), Node()]
        root.children[0].children[1].children = [Node(), Node(), Node()]
        root.children[0].children[2].children = [Node(), Node(), Node()]
        root.children[1].children = [Node(), Node(), Node()]
        root.children[1].children[0].children = [Node(), Node(), Node()]
        root.children[1].children[1].children = [Node(), Node(), Node()]
        root.children[1].children[2].children = [Node(), Node(), Node()]
        root.children[2].children = [Node(), Node(), Node()]
        root.children[2].children[0].children = [Node(), Node(), Node()]
        root.children[2].children[1].children = [Node(), Node(), Node()]
        root.children[2].children[2].children = [Node(), Node(), Node()]

        cvisitor = CountVisitor()
        cvisitor.bfs(root)
        self.root = root
        
    def test_bfs(self):
        visitor = CollectVisitor()
        ans = visitor.bfs(self.root)
        self.assertEqual(ans, [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40])

    def test_dfs_preorder(self):
        visitor = CollectVisitor()
        ans = visitor.dfs(self.root)
        self.assertEqual(ans, [1,2,5,14,15,16,6,17,18,19,7,20,21,22,3,8,23,24,25,9,26,27,28,10,29,30,31,4,11,32,33,34,12,35,36,37,13,38,39,40])

    def test_dfs_inorder(self):
        visitor = CollectVisitor()
        ans = visitor.dfs_inorder(self.root)
        self.assertEqual(ans, [14,5,15,5,16,2,17,6,18,6,19,2,20,7,21,7,22,1,23,8,24,8,25,3,26,9,27,9,28,3,29,10,30,10,31,1,32,11,33,11,34,4,35,12,36,12,37,4,38,13,39,13,40])

    def test_dfs_postorder(self):
        visitor = CollectVisitor()
        ans = visitor.dfs_postorder(self.root)
        self.assertEqual(ans, [14,15,16,5,17,18,19,6,20,21,22,7,2,23,24,25,8,26,27,28,9,29,30,31,10,3,32,33,34,11,35,36,37,12,38,39,40,13,4,1])

    def test_retval_dfs_postorder(self):
        visitor = SumVisitor()
        ans = visitor.dfs_postorder_deque(self.root)
        self.assertEqual(ans, 820)
        visitor = SumVisitor()
        ans = visitor.dfs_postorder_stack(self.root)
        self.assertEqual(ans, 820)

    def test_count_bfs(self):
        cvisitor = CountVisitor()
        ans = cvisitor.bfs(self.root)
        self.assertEqual(ans,40)
        
    def test_count_xbfs(self):
        cvisitor = CountVisitor()
        ans = cvisitor.xbfs(self.root)
        self.assertEqual(ans,40)
        

if __name__ == "__main__":
    unittest.main()



__all__ = ['ExecutableResource']

import os.path
from pyutilib.workflow import resource
import pyutilib.services
import pyutilib.subprocess

# TODO: add support for logging


class ExecutableResource(resource.Resource):

    def __init__(self, name=None, executable=None):
        resource.Resource.__init__(self)
        if name is None and not executable is None:
            name = os.path.basename(executable)
        if executable is None:
            executable = name
        self.register(name, executable)

    def register(self, name, executable):
        self.filename = executable
        if name is None:
            # TBD: when is this branch executed?
            self.description = "Executable" + self.id
            self.name = self.description
        else:
            self.description = "Executable_" + name
            self.name = name
        pyutilib.services.register_executable(executable)

    def run(self, args, logfile=None, debug=False):
        executable = pyutilib.services.registered_executable(self.filename)
        if executable is None:
            # TBD: tests generating this error do not cause global exceptions in a workflow ...
            raise IOError("Cannot find executable '%s'" % self.filename)
        cmd = executable.get_path() + " " + args
        if debug:
            print("Running... %s" % cmd)
        pyutilib.subprocess.run(cmd, outfile=logfile)

    def available(self):
        return not pyutilib.services.registered_executable(
            self.filename) is None



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import sys
from pyutilib.misc import misc

if sys.version_info >= (3, 0):
    xrange = range


def _cross_exec(set_tuple):
    """
    Function used by cross() to generate the cross-product of a tuple
    """
    resulting_set = []
    if len(set_tuple) == 1:
        for val in set_tuple[0]:
            resulting_set.append([val])
    else:
        tmp_set = _cross_exec(set_tuple[1:])
        for val in set_tuple[0]:
            for item in tmp_set:
                #print val, item
                resulting_set.append([val] + item)
    return resulting_set


def cross(set_tuple):
    """
    Returns the cross-product of a tuple of values
    """
    result_set = []
    tmp_set = _cross_exec(set_tuple)
    for val in tmp_set:
        result_set.append(tuple(val))
    return result_set

#def tmp_cross(*args):
#    ans = [[]]
#    for arg in args:
#      ans = [x+[y] for x in ans for y in arg]
#    return ans

if sys.version_info < (3, 0):

    def cross_iter(*sets):
        """
        An iterator function that generates a cross product of
        a set.

        Derived from code developed by Steven Taschuk
        """
        wheels = map(iter, sets)  # wheels like in an odometer
        digits = [it.next() for it in wheels]
        while True:
            yield tuple(digits[:])
            for i in xrange(len(digits) - 1, -1, -1):
                try:
                    digits[i] = wheels[i].next()
                    break
                except StopIteration:
                    wheels[i] = iter(sets[i])
                    digits[i] = wheels[i].next()
            else:
                break

    def flattened_cross_iter(*sets):
        """
        An iterator function that generates a cross product of
        a set, and flattens it.
        """
        wheels = map(iter, sets)  # wheels like in an odometer
        digits = [it.next() for it in wheels]
        ndigits = len(digits)
        while True:
            yield misc.flatten_tuple(tuple(digits[:]))
            for i in xrange(ndigits - 1, -1, -1):
                try:
                    digits[i] = wheels[i].next()
                    break
                except StopIteration:
                    wheels[i] = iter(sets[i])
                    digits[i] = wheels[i].next()
            else:
                break

else:

    def cross_iter(*sets):
        """
        An iterator function that generates a cross product of
        a set.

        Derived from code developed by Steven Taschuk
        """
        wheels = list(map(iter, sets))  # wheels like in an odometer
        digits = [next(it) for it in wheels]
        while True:
            yield tuple(digits[:])
            for i in range(len(digits) - 1, -1, -1):
                try:
                    digits[i] = next(wheels[i])
                    break
                except StopIteration:
                    wheels[i] = iter(sets[i])
                    digits[i] = next(wheels[i])
            else:
                break

    def flattened_cross_iter(*sets):
        """
        An iterator function that generates a cross product of
        a set, and flattens it.
        """
        wheels = list(map(iter, sets))  # wheels like in an odometer
        digits = [next(it) for it in wheels]
        ndigits = len(digits)
        while True:
            yield misc.flatten_tuple(tuple(digits[:]))
            for i in range(ndigits - 1, -1, -1):
                try:
                    digits[i] = next(wheels[i])
                    break
                except StopIteration:
                    wheels[i] = iter(sets[i])
                    digits[i] = next(wheels[i])
            else:
                break



import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__)) + os.sep

import pyutilib.th as unittest


class Tester(unittest.TestCase):

    def test1(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.txt', currdir + 'file2.txt', delete=False)

    @unittest.expectedFailure
    def test2(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt', currdir + 'file2.txt', delete=False)

    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test3(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt', currdir + 'file1.zip', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file1.zip', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file1.zip', currdir + 'file1.zip', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.txt', currdir + 'file2.zip', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.zip', currdir + 'file2.txt', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.zip', currdir + 'file2.zip', delete=False)

    @unittest.skipIf(sys.version_info[:2] >= (3, 0) and sys.version_info[:2] <
                     (3, 3), "Skipping tests with GZ files.")
    def test3gz(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt', currdir + 'file1.txt.gz', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt.gz', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt.gz', currdir + 'file1.txt.gz', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.txt', currdir + 'file2.txt.gz', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.txt.gz', currdir + 'file2.txt', delete=False)
        self.assertFileEqualsBaseline(
            currdir + 'file2.txt.gz', currdir + 'file2.txt.gz', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test4(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt', currdir + 'file3.zip', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test5(self):
        self.assertFileEqualsBaseline(
            currdir + 'file3.zip', currdir + 'file1.txt', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test6(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.zip', currdir + 'file3.txt', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test7(self):
        self.assertFileEqualsBaseline(
            currdir + 'file3.zip', currdir + 'file3.zip', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test8(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.zip', currdir + 'file2.zip', delete=False)

    @unittest.expectedFailure
    def test8gz(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.txt.gz', currdir + 'file2.txt.gz', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] >= (2, 6),
                     "Skipping tests that don't fail.")
    def test9(self):
        self.assertFileEqualsBaseline(
            currdir + 'file1.zip', currdir + 'file2.zip', delete=False)


class TesterL(unittest.TestCase):

    def test1(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.txt', currdir + 'file2.txt', delete=False)

    @unittest.expectedFailure
    def test2(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt', currdir + 'file2.txt', delete=False)

    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test3(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt', currdir + 'file1.zip', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.zip', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.zip', currdir + 'file1.zip', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.txt', currdir + 'file2.zip', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.zip', currdir + 'file2.txt', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.zip', currdir + 'file2.zip', delete=False)

    @unittest.skipIf(sys.version_info[:2] >= (3, 0) and sys.version_info[:2] <
                     (3, 3), "Skipping tests with GZ files.")
    def test3gz(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt', currdir + 'file1.txt.gz', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt.gz', currdir + 'file1.txt', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt.gz', currdir + 'file1.txt.gz', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.txt', currdir + 'file2.txt.gz', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.txt.gz', currdir + 'file2.txt', delete=False)
        self.assertFileEqualsLargeBaseline(
            currdir + 'file2.txt.gz', currdir + 'file2.txt.gz', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test4(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt', currdir + 'file3.zip', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test5(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file3.zip', currdir + 'file1.txt', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test6(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.zip', currdir + 'file3.txt', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test7(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file3.zip', currdir + 'file3.zip', delete=False)

    @unittest.expectedFailure
    @unittest.skipIf(sys.version_info[:2] < (2, 6),
                     "Skipping tests with ZIP files.")
    def test8(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.zip', currdir + 'file2.zip', delete=False)

    @unittest.expectedFailure
    def test8gz(self):
        self.assertFileEqualsLargeBaseline(
            currdir + 'file1.txt.gz', currdir + 'file2.txt.gz', delete=False)


if __name__ == "__main__":
    unittest.main()
